%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Definitions of builtins of module Profile:
%
:- module(prim_profile,
	  [prim_evalTime/4,prim_evalSpace/4]).

:- use_module('../prologbasics').
:- use_module('../basics').

% evaluate the argument to normal form and print the evaluation time:
?- block prim_evalTime(?,?,-,?).
prim_evalTime(T,NF,E0,E) :-
	garbage_collect,
	getRunTime(RTime1),
	getElapsedTime(ETime1),
	statistics(garbage_collection,[GC1,_,_]),
	user:normalizeAndCheck(T,NF,E0,E1),
	getRunTime(RTime2),
	getElapsedTime(ETime2),
	statistics(garbage_collection,[GC2,_,_]),
	writeErr('EVALTIME: Runtime: '),
	RTime is RTime2-RTime1, writeErr(RTime),
	writeErr(' msec.'), nlErr,
	writeErr('EVALTIME: Elapsed time: '),
	ETime is ETime2-ETime1, writeErr(ETime),
	writeErr(' msec.'), nlErr,
	writeErr('EVALTIME: Number of garbage collections: '),
	GC is GC2-GC1, writeErr(GC), nlErr,
	garbageCollectorOn, E1=E.

% evaluate the argument to normal form (without garbage collection)
% and print the evaluation time and memory usage:
?- block prim_evalSpace(?,?,-,?).
prim_evalSpace(T,NF,E0,E) :-
	garbage_collect,
	garbageCollectorOff,
	getRunTime(RTime1),
	getElapsedTime(ETime1),
	statistics(garbage_collection,[GC1,_,_]),
	statistics(global_stack,[GSU1,_]),
	statistics(local_stack,[LSU1,_]),
	user:normalizeAndCheck(T,NF,E0,E1),
	getRunTime(RTime2),
	getElapsedTime(ETime2),
	statistics(garbage_collection,[GC2,_,_]),
	statistics(global_stack,[GSU2,_]),
	statistics(local_stack,[LSU2,_]),
	writeErr('EVALTIME: Runtime: '),
	RTime is RTime2-RTime1, writeErr(RTime),
	writeErr(' msec.'), nlErr,
	writeErr('EVALTIME: Elapsed time: '),
	ETime is ETime2-ETime1, writeErr(ETime),
	writeErr(' msec.'), nlErr,
	writeErr('EVALTIME: Number of garbage collections: '),
	GC is GC2-GC1, writeErr(GC), nlErr,
	writeErr('EVALTIME: Global stack usage: '),
	GS is GSU2-GSU1, writeErr(GS), nlErr,
	writeErr('EVALTIME: Local stack usage: '),
	LS is LSU2-LSU1, writeErr(LS), nlErr,
	garbageCollectorOn, E1=E.
